VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Queue"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'************************************************
'* Queue Class
'* Environment: Visual Basic 6
'* Programmer: Sam Javner
'* Date: 4/29/2007
'*
'* Summary: Implementation of a circular queue of strings
'* Empty condition:   head = tail
'* Full condition:    head = (tail + 1) Mod size
'************************************************



'Explicitly declare all variables
Option Explicit

'Array for the queue, given a size in Initialize
Private q() As String

'Variable to store size of array
Private size As Integer

'Front of queue (remove from)
Private head As Integer

'Tail end of queue (add to)
Private tail As Integer



'************************************************
'* Public Sub Initialize(n As Integer)
'* Summary:     Initializes the queue to a specific size
'* Parameters:  n - size of queue (actually holds n - 1 items)
'************************************************
Public Sub Initialize(n As Integer)

    ReDim q(0 To n - 1)
    
    size = n
    head = 0
    tail = 0

End Sub



'************************************************
'* Public Sub Add(s As String)
'* Summary:     Adds an item to the queue
'* Parameters:  s - string to add
'************************************************
Public Sub Add(s As String)

    If Not IsFull Then
    
        q(tail) = s
        tail = (tail + 1) Mod size
    
    End If

End Sub



'************************************************
'* Public Function Remove() As String
'* Summary:     Removes an item from the queue
'* Returns:     Item from start of queue
'************************************************
Public Function Remove() As String

    If Not IsEmpty Then
   
        Remove = q(head)
        head = (head + 1) Mod size
   
    End If

End Function



'************************************************
'* Public Sub Clear()
'* Summary:     Removes all items from the queue
'************************************************
Public Sub Clear()

    head = tail

End Sub



'************************************************
'* Public Function Count() As Integer
'* Summary:     Returns the number of items in the queue
'* Returns:     Number of items in the queue
'************************************************
Public Function Count() As Integer

    Count = ((tail + size) - head) Mod size

End Function



'************************************************
'* Public Function IsEmpty() As Boolean
'* Summary:     Returns whether or not the queue is empty
'* Returns:     True, if empty
'*              False, if not empty
'************************************************
Public Function IsEmpty() As Boolean

    If head = tail Then IsEmpty = True Else IsEmpty = False

End Function



'************************************************
'* Public Function IsFull() As Boolean
'* Summary:     Returns whether or not the queue is full
'* Returns:     True, if full
'*              False, if not full
'************************************************
Public Function IsFull() As Boolean

    If head = ((tail + 1) Mod size) Then IsFull = True Else IsFull = False

End Function
